/** @file fifo_scen.h
 * This file contains the scenario class declaration.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <fifo_media.h>

#include <ts/scenario.hpp>

#ifdef CPPTESK_AUTOFSM
#include <hw/autofsm.hpp>
#endif

using namespace cpptesk::ts;
using namespace cpptesk::hw;
using namespace cpptesk::tracer;

namespace cpptesk {
namespace examples {
namespace fifo {

/// Common scenario class for this DUT
SCENARIO(FIFOScenario)
{
public:
    // Iteration context type
    typedef IntAbcCtx Ctx;

    /// Scenario initialization function
    virtual bool init(int argc, char ** argv);
    /// Scenario finalization function
    virtual void finish();

    /// Scenario function for no operation (simulate one cycle in DUT and the reference model)
    bool scen_nop(Ctx& ctx);
    /// Scenario function "Place data into FIFO"
    bool scen_push(Ctx& ctx);
    /// Scenario function "Take data from FIFO"
    bool scen_pop(Ctx& ctx);

    /// Function returning the current state of the reference model
    std::string get_state_fifo();

    FIFOScenario(void);
    virtual ~FIFOScenario(void);

protected:
    /// The reference model
    FIFOMediator dut;
#ifdef CPPTESK_AUTOFSM
    AutoFSM<FIFOMediator> autofsm;
#endif
};

/// Child-class of the common scenario class
class ExampleScenario : public FIFOScenario
{
public:
    ExampleScenario();
    virtual ~ExampleScenario();
};

}}}

