/** @file fifo_media.h
 * Header for mediator of FIFO model.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#pragma once

#include <hw/media.hpp>
#include <fifo_model.h>
#include <fifo_interface.h>

#include <string>

namespace cpptesk {
namespace examples {
namespace fifo {

/**
 * @brief Mediator for FIFO model.
 */
CPPTESK_MEDIA(FIFOMediator, FIFO)
{
public:
    FIFOMediator();
    virtual ~FIFOMediator();

    /// Maximum time of waiting of registered reactions (in cycles).
    const static int REACTION_TIMEOUT = 100;

    virtual bool isIface1Ready() const;
    virtual bool isIface2Ready() const;

    /// Serializer of input interface iface1.
    CPPTESK_DECLARE_PROCESS(serialize_iface1);
    /// Serializer of input interface iface2.
    CPPTESK_DECLARE_PROCESS(serialize_iface2);

    /// Deserializer of output interface iface3.
    CPPTESK_DECLARE_PROCESS(deserialize_iface3);

    /// Initialization of test environment.
    virtual void initialize();

    /// Finish of test process.
    virtual void finalize();

    /// Set input values to wires of DUT.
    virtual void setInputs();

    /// Get output values from wires of DUT.
    virtual void getOutputs();

    /// Simulate one cycle of DUT's working.
    virtual void simulate();

private:
    /// Structure for values of input wires of DUT.
    fifo_inputs_t inputs;
    /// Structure for values of output wires of DUT.
    fifo_outputs_t outputs;
};

}}}

