/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#include <hw/event.hpp>
#include <hw/debug.hpp>

#include <hw/diagnostics/error_model.hpp>
#include <hw/diagnostics/reaction_set.hpp>
#include <hw/diagnostics/reaction_unit.hpp>

#include <hw/diagnostics/diagnostics.hpp>
#include <stdio.h>
#include <string>
#include <iterator>
#include "../interface.hpp"

namespace cpptesk {
namespace hw {
namespace diagnostics {

    void Diagnostics::clearReaction()
    {
        _reactionSet.clear();
    }

    void Diagnostics::run()
    {
        // Add to ErrorModel rules
        _errorModel.initialization();

        // Collapse: use normale rule
        if(_reactionSet._reactionVector.size() > 0)
        {
            for(unsigned int i1 = 0; i1<_reactionSet._reactionVector.size(); i1++) //i1 - reaction number 1
            {
                if(_reactionSet._reactionVector[i1].getUseInCollapse() == ::cpptesk::hw::diagnostics::ReactionUnit::REACTION_NOT_USE_IN_COLLAPSE) // 1st reaction not collapsed
                for(unsigned int i2 = 0; i2<_reactionSet._reactionVector.size(); i2++) //j2 - reaction number 2
                { 
                    if((_reactionSet._reactionVector[i2].getUseInCollapse() == ::cpptesk::hw::diagnostics::ReactionUnit::REACTION_NOT_USE_IN_COLLAPSE)
                        &&(_reactionSet._reactionVector[i1].getUseInCollapse() == ::cpptesk::hw::diagnostics::ReactionUnit::REACTION_NOT_USE_IN_COLLAPSE)) // 2nd reaction not collapsed; 1st index != 2nd index
                    {
                        ReactionUnit &r1 = _reactionSet._reactionVector[i1];
                        ReactionUnit &r2 = _reactionSet._reactionVector[i2];
                        
                        ReactionSet reactionSet = _errorModel.transform(r2, r1);
                        _reactionSet.addReaction(reactionSet, i1, i2);
                    }  
                }
            }
        }
        else
        {
            CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Error: reaction set is empty!");
        }

       
        // Collapse: use fuzzy rule
        if(_reactionSet._reactionVector.size() > 0)
        {
            for(unsigned int i1 = 0; i1<_reactionSet._reactionVector.size(); i1++) //i_r_n1 - reaction number 1
            {
                std::vector<int> _error_f_m_main;
                _error_f_m_main.clear();
                _error_f_m_main.push_back(-1);
                _error_f_m_main.push_back(1000);//Rule::FUZZY_MATCH);
                
                std::vector<ReactionUnit> _collapse_reaction;
                std::vector<int> _collapse_reaction_numbers;
                
                if(_reactionSet._reactionVector[i1].getUseInCollapse() == ::cpptesk::hw::diagnostics::ReactionUnit::REACTION_NOT_USE_IN_COLLAPSE) // 1st reaction not collapsed
                for(unsigned int i2 = 0; i2<_reactionSet._reactionVector.size(); i2++) //j_r_n2 - reaction number 2
                { 
                    if((_reactionSet._reactionVector[i2].getUseInCollapse() == ::cpptesk::hw::diagnostics::ReactionUnit::REACTION_NOT_USE_IN_COLLAPSE)
                        &&(_reactionSet._reactionVector[i1].getUseInCollapse() == ::cpptesk::hw::diagnostics::ReactionUnit::REACTION_NOT_USE_IN_COLLAPSE)) // 2nd reaction not collapsed; 1st index != 2nd index
                    {
                        ReactionUnit &r1 = _reactionSet._reactionVector[i1];
                        ReactionUnit &r2 = _reactionSet._reactionVector[i2];
                        
                        std::vector<int> _error_f_m = _errorModel.fuzzyMatch(r2._reaction, r1._reaction);
                        if(_error_f_m[1]<_error_f_m_main[1])
                        {
                            _error_f_m_main.clear();
                            _error_f_m_main = _error_f_m;
                            _collapse_reaction_numbers.clear();
                            _collapse_reaction_numbers.push_back(i1);
                            _collapse_reaction_numbers.push_back(i2);
                            _collapse_reaction.clear();
                            _collapse_reaction.push_back(r1);
                            _collapse_reaction.push_back(r2);
                        }
                        
                    }  
                }
                
                if(_error_f_m_main[1] < 1000)//cpptesk::hw::diagnostics::Rule::FUZZY_MATCH)
                {    
                    ReactionSet reactionSet = _errorModel.fuzzyTransform(_collapse_reaction[0], _collapse_reaction[1], _error_f_m_main[0]);
                    _reactionSet.addReaction(reactionSet, _collapse_reaction_numbers[0], _collapse_reaction_numbers[1]);
                    _reactionSet._reactionVector[_collapse_reaction_numbers[0]].useInCollapse();
                    _reactionSet._reactionVector[_collapse_reaction_numbers[0]].setCollapseRule(_error_f_m_main[0]);
                    _reactionSet._reactionVector[_collapse_reaction_numbers[1]].useInCollapse();
                    _reactionSet._reactionVector[_collapse_reaction_numbers[1]].setCollapseRule(_error_f_m_main[0]);
                }
            }
        }
        else
        {
            CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Error: reaction set is empty!");
        }

        this->printVector();
        //this->printFullVector();
        
        clearReaction();
        _errorModel.removeRules();
    }    
    
    void Diagnostics::printVector()
    {
        // CPPTESK_SET_DEBUG_LEVEL(cpptesk::hw::DEBUG_TOOL, true);

        bool spoiler_shown = false;
        for(unsigned int i = 0; i<_reactionSet._reactionVector.size(); i++)
        {
            if(_reactionSet._reactionVector[i]._reaction.isError())
            {
                std::vector<int> _parents = _reactionSet._reactionVector[i].getParent();
                if(_parents.size() == 0) continue;

                if(!spoiler_shown)
                {
                    debug_style_t style = Debug::get().getMessageStyle(DEBUG_INFO); 
                    CPPTESK_COLORED_DEBUG_PRINT(DEBUG_ERROR, style.fg_color, style.bg_color, "Error diagnostics: ");
                    spoiler_shown = true; 
                }

                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Reaction number in the vector: " << i);
                
                if(_reactionSet._reactionVector[i]._reaction.getError() == MISSING_REACTION)
                {
                    CPPTESK_DEBUG_PRINT(DEBUG_ERROR, _reactionSet._reactionVector[i]._reaction.getShortDescription() << " SpecMessage: " <<
                                        _reactionSet._reactionVector[i]._reaction.getSpecMessage()->forceToString() << std::endl);
                }
                else if (_reactionSet._reactionVector[i]._reaction.getError() == UNEXPECTED_REACTION)
                {
                    CPPTESK_DEBUG_PRINT(DEBUG_ERROR, _reactionSet._reactionVector[i]._reaction.getShortDescription() << " ImplMessage: " <<
                                        _reactionSet._reactionVector[i]._reaction.getImplMessage()->forceToString() << std::endl);
                }
                else
                {
                    CPPTESK_DEBUG_PRINT(DEBUG_ERROR, _reactionSet._reactionVector[i]._reaction.getShortDescription() << std::endl <<
                    " SpecMessage: " << _reactionSet._reactionVector[i]._reaction.getSpecMessage()->forceToString() << std::endl <<
                    " ImplMessage: " << _reactionSet._reactionVector[i]._reaction.getImplMessage()->forceToString() << std::endl);
                }

                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Collapsed: " << (_reactionSet._reactionVector[i].getUseInCollapse() == 1 ? "Yes" : "No") << ", " <<
                                                "Collapse Rule: " << _reactionSet._reactionVector[i].getCollapseRule());

                if(_reactionSet._reactionVector[i].getUseInCollapse() == 1)
                {
                   // CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "RuleDiscription:    " << _errorModel.getFuzzyRuleDescription(_reactionSet._reactionVector[i].getCollapseRule()));
                }
                if(_parents.size() != 0)
                {
                    CPPTESK_DEBUG_PRINT(DEBUG_TOOL,"Reaction Parents:");
                    for(unsigned int j=0; j<_parents.size(); j++)
                    {
                        CPPTESK_DEBUG_PRINT(DEBUG_TOOL, " " << _parents[j]);
                        
                        ReactionUnit &reactionParent = _reactionSet._reactionVector[_parents[j]];
                        
                        // Print parents interface
                        const ::cpptesk::hw::Interface *iface = reactionParent._reaction.where();
                        CPPTESK_DEBUG_PRINT(DEBUG_ERROR, "Parent " << (j+1) << " interface: " << iface->getName());
                        
                                            if(reactionParent._reaction.getError() == MISSING_REACTION)
                            {
                                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, reactionParent._reaction.getShortDescription() << " SpecMessage: " <<
                                                   reactionParent._reaction.getSpecMessage()->forceToString() << std::endl);
                            }
                            else if (reactionParent._reaction.getError() == UNEXPECTED_REACTION)
                            {
                                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, reactionParent._reaction.getShortDescription() << " ImplMessage: " <<
                                                    reactionParent._reaction.getImplMessage()->forceToString() << std::endl);
                            }
                            else
                            {
                                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, reactionParent._reaction.getShortDescription() <<
                                    " SpecMessage: " << reactionParent._reaction.getSpecMessage()->forceToString() <<
                                    " ImplMessage: " << reactionParent._reaction.getImplMessage()->forceToString() << std::endl);
                            }

                        CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Collapsed: " << (reactionParent.getUseInCollapse() == 1 ? "Yes" : "No") << ", " <<
                                                "Collapse Rule: " << reactionParent.getCollapseRule());
                                    
                        
                    }
                }
                _parents.clear();
            }
        }
    }    
    
    //Special for debug
    void Diagnostics::printFullVector()
    {
        CPPTESK_SET_DEBUG_LEVEL(cpptesk::hw::DEBUG_TOOL, true);

       // CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "=========== NEW TEST VECTOR ==============");
        debug_style_t style = Debug::get().getMessageStyle(DEBUG_INFO); 
        CPPTESK_COLORED_DEBUG_PRINT(DEBUG_TOOL, style.fg_color, style.bg_color, "Error diagnostics: ");

        for(unsigned int i = 0; i<_reactionSet._reactionVector.size(); i++)
        {
            CPPTESK_DEBUG_PRINT(DEBUG_TOOL, ".");
            CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "#" << i << "=================================");
            CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Reaction / Vector Size = " << i << "/" <<_reactionSet._reactionVector.size());
              
//            if(_reactionSet._reactionVector[i]._reaction.isError())
//            {
                std::vector<int> _parents = _reactionSet._reactionVector[i].getParent();
              //  if(_parents.size() == 0) continue;

              //  CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Reaction number in the vector: " << i);
                
               // CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "+");
                
                if(_reactionSet._reactionVector[i]._reaction.getError() == MISSING_REACTION)
                {
                 //   CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "MISSING Branch");
                    CPPTESK_DEBUG_PRINT(DEBUG_TOOL, _reactionSet._reactionVector[i]._reaction.getShortDescription() << " SpecMessage: " <<
                                        _reactionSet._reactionVector[i]._reaction.getSpecMessage()->forceToString() << std::endl);
                }
                else if (_reactionSet._reactionVector[i]._reaction.getError() == UNEXPECTED_REACTION)
                {
                   // CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "UNEXPECTED Branch");
                    CPPTESK_DEBUG_PRINT(DEBUG_TOOL, _reactionSet._reactionVector[i]._reaction.getShortDescription() << " ImplMessage: " <<
                                        _reactionSet._reactionVector[i]._reaction.getImplMessage()->forceToString() << std::endl);
                }
                else
                {
                //    CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "ELSE(INCORRECT) Branch");
                    CPPTESK_DEBUG_PRINT(DEBUG_TOOL, _reactionSet._reactionVector[i]._reaction.getShortDescription() << std::endl <<
                    " SpecMessage: " << _reactionSet._reactionVector[i]._reaction.getSpecMessage()->forceToString() << std::endl <<
                    " ImplMessage: " << _reactionSet._reactionVector[i]._reaction.getImplMessage()->forceToString() << std::endl);
                }

                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Collapsed: " << (_reactionSet._reactionVector[i].getUseInCollapse() == 1 ? "Yes" : "No") << ", " <<
                                                "Collapse Rule: " << _reactionSet._reactionVector[i].getCollapseRule());

                if(_reactionSet._reactionVector[i].getUseInCollapse() == 1)
                {
                   // CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "RuleDiscription:    " << _errorModel.getFuzzyRuleDescription(_reactionSet._reactionVector[i].getCollapseRule()));
                }
                if(_parents.size() != 0)
                {
                    CPPTESK_DEBUG_PRINT(DEBUG_TOOL,"Reaction Parents:");
                    for(unsigned int j=0; j<_parents.size(); j++)
                    {
                        CPPTESK_DEBUG_PRINT(DEBUG_TOOL, " " << _parents[j]);
                        
                        ReactionUnit &reactionParent = _reactionSet._reactionVector[_parents[j]];
                        
                        // Print parents interface
                        const ::cpptesk::hw::Interface *iface = reactionParent._reaction.where();
                        CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Parent " << (j+1) << " interface: " << iface->getName());
                        
                            if(reactionParent._reaction.getError() == MISSING_REACTION)
                            {
                                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, reactionParent._reaction.getShortDescription() << " SpecMessage: " <<
                                                   reactionParent._reaction.getSpecMessage()->forceToString() << std::endl);
                            }
                            else if (reactionParent._reaction.getError() == UNEXPECTED_REACTION)
                            {
                                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, reactionParent._reaction.getShortDescription() << " ImplMessage: " <<
                                                    reactionParent._reaction.getImplMessage()->forceToString() << std::endl);
                            }
                            else
                            {
                                CPPTESK_DEBUG_PRINT(DEBUG_TOOL, reactionParent._reaction.getShortDescription() <<
                                    " SpecMessage: " << reactionParent._reaction.getSpecMessage()->forceToString() <<
                                    " ImplMessage: " << reactionParent._reaction.getImplMessage()->forceToString() << std::endl);
                            }

                        CPPTESK_DEBUG_PRINT(DEBUG_TOOL, "Collapsed: " << (reactionParent.getUseInCollapse() == 1 ? "Yes" : "No") << ", " <<
                                                "Collapse Rule: " << reactionParent.getCollapseRule());
                                    
                        
                    }
                }
                _parents.clear();
            //}
        }
    }
}}}
