/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#pragma once
#include "hw/event.hpp"

#include <string>
#include <vector>

#include "rule/rule.hpp"

#include "rule/correct.hpp"
#include "rule/associative.hpp"
#include "rule/union.hpp"
#include "rule/spec_intersection.hpp"
#include "rule/impl_intersection.hpp"
#include "rule/incomplete_associative.hpp"

#include "rule/fuzzy_associative.hpp"
#include "rule/fuzzy_spec_intersection.hpp"
#include "rule/fuzzy_impl_intersection.hpp"
#include "rule/fuzzy_union.hpp"

namespace cpptesk {
namespace hw {
namespace diagnostics {

class ErrorModel: 
      public Rule
{
    public: 
        ErrorModel() {};
        virtual ~ErrorModel() {};

    unsigned int priority;  
    std::vector<Rule*> ruleSet; // for normal rules
    std::vector<Rule*> fuzzyRuleSet; // for fuzzy rules
  
    /*ErrorModel::ErrorModel()
    {  //Empty
    }

    ErrorModel::~ErrorModel()
    {  //Empty
    }*/

    void addRule(Rule *_rule)
    // Add rule to the ruleSet 
    {
        ruleSet.push_back(_rule);
    }

    void addFuzzyRule(Rule *_fuzzyRule)
    // Add rule to the fuzzyRuleSet 
    {
        fuzzyRuleSet.push_back(_fuzzyRule);
    }

    void initialization()
    {
        // Create normal rule set        
        CorrectReactionEliminationRule *_rule_1 = new CorrectReactionEliminationRule();
        addRule(_rule_1);
        AssociativeEliminationRule *_rule_2 = new AssociativeEliminationRule();
        addRule(_rule_2);
        UnionEliminationRule *_rule_3 = new UnionEliminationRule();
        addRule(_rule_3);
        SpecIntersectionEliminationRule *_rule_4 = new SpecIntersectionEliminationRule();
        addRule(_rule_4);
        ImplIntersectionEliminationRule *_rule_5 = new ImplIntersectionEliminationRule();
        addRule(_rule_5);
        IncompleteAssociativeEliminationRule *_rule_6 = new IncompleteAssociativeEliminationRule();
        addRule(_rule_6);
        
        // Create fuzzy rule set        
        FuzzyAssociativeEliminationRule  *_fuzzyRule_1 = new FuzzyAssociativeEliminationRule();
        addFuzzyRule(_fuzzyRule_1);
        FuzzyUnionEliminationRule  *_fuzzyRule_2 = new FuzzyUnionEliminationRule();
        addFuzzyRule(_fuzzyRule_2);
        //FuzzyImplIntersectionEliminationRule  *_fuzzyRule_3 = new FuzzyImplIntersectionEliminationRule();
        //addFuzzyRule(_fuzzyRule_3);
        //FuzzySpecIntersectionEliminationRule  *_fuzzyRule_4 = new FuzzySpecIntersectionEliminationRule();
        //addFuzzyRule(_fuzzyRule_4);
    }

    void removeRules()
    {
        ruleSet.clear();
        fuzzyRuleSet.clear();
    }

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2)     // Collapse the reactions
    {
        int _match_rule = this->match(_reactionUnit_1._reaction, _reactionUnit_2._reaction);
        
        ReactionSet _transform_out;

        if(_match_rule != NOT_FOUND_COLLAPSE_RULE)
        {
            Rule* _rule;

            _rule = ruleSet[_match_rule]; 
            
            _transform_out = _rule->transform(_reactionUnit_1, _reactionUnit_2);
            _rule->setCollapseRule(_reactionUnit_1, _reactionUnit_2, _match_rule);
        }
        
        return _transform_out; 
    }

    ReactionSet fuzzyTransform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, 
                               ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2,
                               int _fuzzy_match_rule)     // Collapse the reactions
    {
        //int _fuzzy_match_rule = this->fuzzyMatch(_reactionUnit_1._reaction, _reactionUnit_2._reaction);
        
        ReactionSet _transform_out;

        //if(_fuzzy_match_rule != NOT_FOUND_COLLAPSE_RULE)
        //{
            Rule* _fuzzyRule;

            _fuzzyRule = fuzzyRuleSet[_fuzzy_match_rule]; 
            
            _transform_out = _fuzzyRule->transform(_reactionUnit_1, _reactionUnit_2);
            _fuzzyRule->setCollapseRule(_reactionUnit_1, _reactionUnit_2, _fuzzy_match_rule);
            _reactionUnit_1.useInCollapse();
        //}
        
        return _transform_out; 
    }

    std::string getRuleDescription(int _rule_index)
    {
        Rule* _rule;
        _rule = ruleSet[_rule_index]; 

        std::string _description = _rule->getRuleDescription();
        return _description;
    }

    std::string getFuzzyRuleDescription(int ruleIndex)
    {
        Rule* rule;
        rule = fuzzyRuleSet[ruleIndex]; 

        std::string _description = rule->getRuleDescription();
        return _description;
    }


    int match(::cpptesk::hw::Reaction &_reaction_1, ::cpptesk::hw::Reaction &_reaction_2)
    // Compare the reactions
    {
        int _match_value = FUZZY_MATCH;
        int i_rule = NOT_FOUND_COLLAPSE_RULE;
        for(unsigned int i=0; i<ruleSet.size(); i++)
        {
             Rule* _rule = ruleSet[i];
             int _match_value_new = _rule->match(_reaction_1, _reaction_2);
             if(_match_value_new < _match_value)
             {
                 _match_value = _match_value_new;
                 i_rule = i; //found collapse rule
             }

        }    
        return i_rule;
    }

    std::vector<int> fuzzyMatch(::cpptesk::hw::Reaction &_reaction_1, ::cpptesk::hw::Reaction &_reaction_2)
    // Compare the reactions
    {
        int _match_value = FUZZY_MATCH;
        int i_rule = NOT_FOUND_COLLAPSE_RULE;
        
        std::vector<int> _fuzzyMatchresult;
        
        for(unsigned int i=0; i<fuzzyRuleSet.size(); i++)
        {
             Rule* _fuzzyRule = fuzzyRuleSet[i];
             int _match_value_new = _fuzzyRule->match(_reaction_1, _reaction_2);
             if(_match_value_new < _match_value)
             {
                 _match_value = _match_value_new;
                 i_rule = i; //found collapse rule
             }

        }    
        
        _fuzzyMatchresult.clear();
        _fuzzyMatchresult.push_back(i_rule);
        _fuzzyMatchresult.push_back(_match_value);
        
        return _fuzzyMatchresult;
    }

    void weight()
    // Get the weight of operation
    {
    }

    void distance()
    //  
    {
    }
};

}}}
