/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/event.hpp>

#include <vector>

namespace cpptesk {
namespace hw {
namespace diagnostics {

class ReactionUnit //ReactionUnit
{
    public:
    const static int REACTION_NOT_USE_IN_COLLAPSE = 0; 
    const static int REACTION_USE_IN_COLLAPSE = 1; 
        
    ReactionUnit(const ::cpptesk::hw::Reaction &_r) : _reaction(_r)
    {
        _childReaction1 = -1;
        _parentReaction1 = -1;
        _childReaction2 = -1;
        _parentReaction2 = -1;
        _useInCollapse = REACTION_NOT_USE_IN_COLLAPSE;    
        _collapseRule = -1;
    }
    
    ReactionUnit(::cpptesk::hw::Reaction &_r, int _parent1, int _parent2) : _reaction(_r)
    {
        _childReaction1 = -1;
        _childReaction2 = -1;
        _useInCollapse = REACTION_NOT_USE_IN_COLLAPSE;    
        _collapseRule = -1;
        this->setParent(_parent1, _parent2);
    }
    
    virtual ~ReactionUnit() {}
    
    void useInCollapse()
    {
        _useInCollapse = REACTION_USE_IN_COLLAPSE;
    }
  
    int getUseInCollapse()
    {
        return _useInCollapse;
    }

    void setCollapseRule(int _rule)
    {
        _collapseRule = _rule;
    }

    int getCollapseRule()
    {
        return _collapseRule;
    }

    void setParent(int _parent1, int _parent2)
    {
        _parentReaction1 = _parent1;
        _parentReaction2 = _parent2;
    }

    std::vector<int> getParent()
    {
        std::vector<int> _parents;
        
        if(this->_parentReaction1 != -1)
            _parents.push_back(this->_parentReaction1);
        if(this->_parentReaction2 != -1)
            _parents.push_back(this->_parentReaction2);
        
        return  _parents;
    }
    
    ::cpptesk::hw::Reaction _reaction;
    
    protected:
        int _useInCollapse;
        int _childReaction1, _childReaction2;
        int _parentReaction1, _parentReaction2;
        int _collapseRule;
};

}}}
