/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Spec Intersection Elimination 
//           (Rule # 5 in C++TESK.ErrorDiagnostics.ru.doc):
//           {[a:b]+[b:x] => [b:b]+[a:x]}
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/diagnostics/rule/rule.hpp>
#include <hw/diagnostics/transform_unit.hpp>

#include <stdio.h>


namespace cpptesk {
namespace hw {
namespace diagnostics {

class SpecIntersectionEliminationRule: 
      public Rule
{
    public:

    SpecIntersectionEliminationRule() {}
    virtual ~SpecIntersectionEliminationRule() {};

    ReactionSet transform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2);
    // Collapse the reactions

    int match(::cpptesk::hw::Reaction _reaction_1, ::cpptesk::hw::Reaction _reaction_2);
    // Compare the reactions

    void weight()
    // Get the weight of operation
    {
    }
    
    void setCollapseRule(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2, int _rule)
    // Set rule number
    {
        _reactionUnit_1.setCollapseRule(_rule);
        _reactionUnit_2.setCollapseRule(_rule);
    }

    void distance()
    //  
    {
    }
    
    std::string getRuleDescription()
    {
        return "\n Spec Intersection Elimination Rule - use for INCORRECT and MISSING reactions: {[a:b]+[b:x] => [b:b]+[a:x]}.";
    }

};

}}}
