/*Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Protsenko (protsenko@ispras.ru)
// SYNOPSIS: Union Elimination 
//           (Rule # 4 in C++TESK.ErrorDiagnostics.ru.doc):
//           {[a:x]+[x:a] => [a:a]+[x:x]}
///////////////////////////////////////////////////////////////////////////////

#include <hw/diagnostics/rule/rule.hpp>
#include <hw/diagnostics/transform_unit.hpp>
#include <hw/diagnostics/reaction_set.hpp>
#include <hw/diagnostics/rule/union.hpp>

#include <hw/errors.hpp>

#include <stdio.h>
namespace cpptesk {
namespace hw {
namespace diagnostics {

    ReactionSet UnionEliminationRule::transform(::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_1, ::cpptesk::hw::diagnostics::ReactionUnit &_reactionUnit_2)    // Collapse the reactions
    {
        ReactionSet _transform_out;
        
        const ::cpptesk::hw::Message *_message1Spec;
        const ::cpptesk::hw::Message *_message2Impl;
 
        const ::cpptesk::hw::Interface *iface1;
                
        if((_reactionUnit_1._reaction.getError() == MISSING_REACTION)&&(_reactionUnit_2._reaction.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            _message1Spec = _reactionUnit_1._reaction.getSpecMessage();
            // Message from reaction 2
            _message2Impl = _reactionUnit_2._reaction.getImplMessage();
            
            iface1 = _reactionUnit_1._reaction.where();
        }
        else
        {
            // Message from reaction 1
            _message1Spec = _reactionUnit_2._reaction.getSpecMessage();
            // Message from reaction 2
            _message2Impl = _reactionUnit_1._reaction.getImplMessage();
            
            iface1 = _reactionUnit_2._reaction.where();
        }

        // Get "start" and "end" time of reactions
        cycle_t start_time   =  _reactionUnit_1._reaction.whenStarted();
        cycle_t start_time_2 =  _reactionUnit_2._reaction.whenStarted();
        cycle_t end_time     =  _reactionUnit_1._reaction.whenStopped();
        cycle_t end_time_2   =  _reactionUnit_2._reaction.whenStopped();
        
        if(start_time > start_time_2) start_time = start_time_2; // Min start time
        if(end_time < end_time_2)     end_time = end_time_2;     // Max end time
        
        ::cpptesk::hw::Reaction reactionNormal = Reaction(NORMAL_REACTION, *iface1, *_message1Spec, *_message2Impl, start_time, end_time, "normal reaction from rule#4");
        
        _transform_out.addReaction(reactionNormal);
        
        // Set flag
        _reactionUnit_1.useInCollapse();
        _reactionUnit_2.useInCollapse();
       
        return _transform_out;
    }

    int UnionEliminationRule::match(::cpptesk::hw::Reaction _reaction_1, ::cpptesk::hw::Reaction _reaction_2)
    // Compare the reactions
    {
        int _match = Rule::FALSE_MATCH;

        if((_reaction_1.getError() == MISSING_REACTION)&&(_reaction_2.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            const ::cpptesk::hw::Message *_message1Spec = _reaction_1.getSpecMessage();
            // Message from reaction 2
            const ::cpptesk::hw::Message *_message2Impl = _reaction_2.getImplMessage();
    
            if(_message1Spec->size() == _message2Impl->size())
            {
                _match = EXACT_MATCH;

                for(::cpptesk::hw::Message::const_iterator i = _message1Spec->begin(), j = _message2Impl->begin();
                    i != _message1Spec->end(); i++, j++)
                {
                    const Field<> &field_Spec = (*_message1Spec).**i;
                    const Field<> &field_Impl = (*_message2Impl).**j;

                    if(!field_Spec.equals(field_Impl))
                    {
                        _match = FUZZY_MATCH;
                    }
                    
                }
            }
        }

        if((_reaction_2.getError() == MISSING_REACTION)&&(_reaction_1.getError() == UNEXPECTED_REACTION))
        {
            // Message from reaction 1
            const ::cpptesk::hw::Message *_message1Impl = _reaction_1.getImplMessage();
            // Message from reaction 2
            const ::cpptesk::hw::Message *_message2Spec = _reaction_2.getSpecMessage();
            
            if(_message2Spec->size() == _message1Impl->size())
            {
                _match = EXACT_MATCH;

                for(::cpptesk::hw::Message::const_iterator i = _message2Spec->begin(), j = _message1Impl->begin();
                    i != _message2Spec->end(); i++, j++)
                {
                    const Field<> &field_Spec = (*_message2Spec).**i;
                    const Field<> &field_Impl = (*_message1Impl).**j;

                    if(!field_Spec.equals(field_Impl))
                    {
                        _match = FUZZY_MATCH;
                    }
                    
                }
            }
        }
        
        return _match;
    }

}}}
