/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Predicate event 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/sequence/event.hpp>

#define CPPTESK_DECLARE_EVENT(event_name, event_predicate) \
    /* Declare the predicate method */ \
    bool event_name () const { return event_predicate; }

#define CPPTESK_DECLARE_ACTION(event_name, event_action) \
    /* Declare the action method */ \
    void event_name () { event_action; }

#define CPPTESK_EVENT(event_name) ::cpptesk::hw::sequence::PredicateEvent<model_t>(*this, &model_t::event_name)

#define CPPTESK_ACTION(event_name) ::cpptesk::hw::sequence::ActionEvent<model_t>(*this, &model_t::event_name)

#ifdef HW_SHORT_NAMES
    #define DECLARE_EVENT(event_name, event_predicate) CPPTESK_DECLARE_EVENT(event_name, event_predicate)
    #define DECLARE_ACTION(event_name, event_action)   CPPTESK_DECLARE_ACTION(event_name, event_action)
    #define EVENT(event_name)                          CPPTESK_EVENT(event_name)
    #define ACTION(event_name)                         CPPTESK_ACTION(event_name)
#endif // HW_SHORT_NAMES

namespace cpptesk {
namespace hw {
namespace sequence {

template<class UserModel>
class PredicateEvent: public Event
{
public:
    PredicateEvent(const UserModel &model, bool (UserModel::*predicate) () const):
        _model(model), _predicate(predicate) {}

    PredicateEvent(const PredicateEvent &rhs): Event(rhs), _model(rhs._model), _predicate(rhs._predicate) {}

    virtual ~PredicateEvent() {}

    bool occured() const { return (_model.*_predicate)(); }

protected:
    const UserModel &_model;
    bool (UserModel::*_predicate)() const;
};

template<class UserModel>
class ActionEvent: public Event
{
public:
    ActionEvent(UserModel &model, void (UserModel::*action) ()):
        _model(model), _action(action) {}

    ActionEvent(const ActionEvent &rhs): Event(rhs), _model(rhs._model), _action(rhs._action) {}

    virtual ~ActionEvent() {}

    void handler() { (_model.*_action)(); }

protected:
    UserModel &_model;
    void (UserModel::*_action)();
};

}}} // namespace cpptesk::hw::sequence

