/*
 * Copyright 2010-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Shared pointer
///////////////////////////////////////////////////////////////////////////////

#pragma once

namespace cpptesk {
namespace hw {

template<class T>
class SharedPtr
{
public:
    SharedPtr(T* ptr = 0): _counter(new RefCounter(ptr))
        { _counter->r(); }

    SharedPtr(const SharedPtr &rhs): _counter(rhs._counter)
        { _counter->r(); }

   ~SharedPtr() { _counter->d(); }

    SharedPtr& operator = (const SharedPtr &rhs)
    {
        if(_counter != rhs._counter)
        {
            _counter->d();
            _counter = rhs._counter;
            _counter->r();
        }

        return *this;
    }

    T *operator -> () const { return  (_counter->_ptr); }
    T &operator *  () const { return *(_counter->_ptr); }

    bool isNull() const { return _counter->_ptr == 0; }

    const T *get() const { return _counter->_ptr; }

private:
    class RefCounter
    {
    public:
        T* _ptr;

        void r() { _ref++; }
        void d() { if(_ref > 0 && --_ref == 0) { delete this; } }

        RefCounter(T* ptr): _ptr(ptr), _ref(0) {}
       ~RefCounter() { delete _ptr; }

    private:
        int _ref;
    };

    RefCounter *_counter;
};

}} // cpptesk::hw

