/*
 * Copyright 2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * File: aspectrace_internals.h
 * Description:
 *       This file contains the definition of Aspectrace low level
 *       tracing interface used by aspect libraries.
 */

#ifndef __ASPECTRACE_INTERNALS_H
#define __ASPECTRACE_INTERNALS_H

#include <atl/map.h>
#include <utils/boolean.h>

#include "aspectrace.h"

/* If defined, all C-strings are duplicated in internal data structures */
/* #define ALLOC_NEW */


typedef struct TraceNode {
	const char *s; /* either name or text */
	Map *attrs;
	struct TraceNode *firstChild;
	struct TraceNode *nextSibling;
	bool isText;
} TraceNode;

// /** Create regular TraceNode
//  * @param children NULL-terminated array of nodes */
// TraceNode *create_TraceNode_a(const char *name, const char **attrs, TraceNode **children);

/** Create regular TraceNode
 * @param children NULL-terminated list of nodes */
TraceNode *create_TraceNode(const char *name, const char **attrs, TraceNode **children, size_t count);

/** Count the lengh of a NULL-terminated array
 * @param a pointer to the first element of a NULL-terminated array
 */
size_t list_len(const void** list);

/* Create textual TraceNode */
TraceNode *create_stringTraceNode(String *text);

/* Create textual TraceNode */
TraceNode *create_textTraceNode(const char *text);

/* Normally should be called only by aspect_trace() */
void destroy_TraceNode(TraceNode *node);


/* Check filters before generating trace node structures */
bool aspectrace_allowed(const char *aspect, const char *message /*NULL for tag*/);


const char *to_str_MessageLevel( enum MessageLevel level );

void aspect_trace(const char *aspect, const char *channel, TraceNode *node);

#endif /* __ASPECTRACE_INTERNALS_H */
