/**** API for synchronizer. ****/

#ifndef __NETFSM_SYNCHRONIZER_H

#define __NETFSM_SYNCHRONIZER_H

#include <atl/map.h>
#include <utils/boolean.h>

#include <generator.h>


/*** Create/destroy Synchronizer. Not Object !!! ***/

typedef struct Synchronizer Synchronizer;

Synchronizer *create_Synchronizer( TE_Graph *g, TestEngine *t );
void destroy_Synchronizer( Synchronizer *sync );


/*** Configuration. All methods return 0 on success, some failure code otherwise. ***/

/** Configure the input port and set the number of expected incoming connections.
 Input port is opened immediately. */
int sync_SetInputConf( Synchronizer *sync, unsigned short inPort, int numInConnections );
/** Add an outgoing connection. */
int sync_AddOutConnection( Synchronizer *sync, char *outHost, unsigned short outPort );

int sync_TolerateNeighbourDeaths( Synchronizer *sync, bool tolerate );

/*** Save and load log. Effective only for active synchronizer. ***/
int sync_SaveLog( Synchronizer *sync, const char *fileName );
/* @param thenAppend if set to true and save log not set, then
 the following log will be appended to the same file after loading.
 @param simLoad Set to true if the same log is loaded simultaneously by
 all nodes. */
int sync_LoadLog( Synchronizer *sync, const char *fileName, bool thenAppend, bool simLoad );

/** Notify the synchronizer that all in and out connections are configured,
 all peers have received their configuration and opened their input ports.
 Implementation tries to establish all needed connections. */
int sync_ConfigurationFinished( Synchronizer *sync );


/*** Control flow. ***/

/*** For active synchronizer with its own control thread ***/
void sync_SetExchangeDelay( Synchronizer *sync, int msecs );
int sync_StartThread( Synchronizer *sync );
/* Returns after the synchronizer thread have shut down */
int sync_FinishThread( Synchronizer *sync );

/*** For passive synchronizer relying on OS buffers and regular calls. ***/
bool sync_DoExchange( Synchronizer *sync );
void sync_Shutdown( Synchronizer *sync );


/*** Statistics. ***/
String* getAttrValue_Synchronizer( Synchronizer *sync, const char *attr_name );
Map* getAllAttrValues_Synchronizer( Synchronizer *sync );

#define SYNC_ATTR_STATUS		"status"
#define SYNC_ATTR_STATUS_INIT	"init"
#define SYNC_ATTR_STATUS_SEMI_CONF	"semi_configured"
#define SYNC_ATTR_STATUS_READY		"ready"
#define SYNC_ATTR_STATUS_RUNNING	"running"
#define SYNC_ATTR_STATUS_SHUTDOWN	"shutting_down"
#define SYNC_ATTR_STATUS_FINALIZED	"finalized"

#define SYNC_ATTR_IN_CONNECTIONS	"in_connections"
#define SYNC_ATTR_IN_FAILURES		"in_connections_failed"
#define SYNC_ATTR_OUT_CONNECTIONS	"out_connections"
#define SYNC_ATTR_OUT_FAILURES		"out_connections_failed"
#define SYNC_ATTR_OPEN_TRANSACTIONS	"open_transactions"
#define SYNC_ATTR_COMPLETED_TRANSACTIONS	"completed_transactions"
#define SYNC_ATTR_OUT_QUEUE_SIZE	"out_queue_size"
#define SYNC_ATTR_LOGGING_STATUS	"logging_status"
#define SYNC_ATTR_LOG_LOADED_ARCS	"loaded_log_arcs"
#define SYNC_ATTR_LOG_SAVED_ARCS	"saved_log_arcs"


#endif /* __NETFSM_SYNCHRONIZER_H */
